@@ -47,7 +47,7 @@ except:
 
 PLUGIN_NAME = _("Music Sharing")
 PLUGIN_AUTHORS = ['Aren Olson <reacocard@gmail.com>']
-PLUGIN_VERSION = '0.7.9'
+PLUGIN_VERSION = '0.8.1'
 PLUGIN_DESCRIPTION = _(r"""Allows playing of DAAP music shares.
 \n\nDepends: python-daap, python-avahi.""")
 
@@ -266,6 +266,8 @@ class NetworkPanel(collection.CollectionPanel):
         self.transfer_queue = None
         self.transferring = False
         self.queue = None
+        self.model = gtk.TreeStore(gtk.gdk.Pixbuf, object, str)
+        self.model_blank = gtk.TreeStore(gtk.gdk.Pixbuf, object, str)
 
         self.keyword = None
         self.track_cache = dict()
@@ -282,6 +284,8 @@ class NetworkPanel(collection.CollectionPanel):
             xl.path.get_data('images', 'track.png'))
         self.genre_image = gtk.gdk.pixbuf_new_from_file(
             xl.path.get_data('images', 'genre.png'))
+        self.year_image = gtk.gdk.pixbuf_new_from_file(xl.path.get_data(
+            'images', 'year.png'))
 
         self.setup_widgets()
 
