@@ -15,40 +15,20 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 
-import httplib, md5, os, re, threading, time, urllib, urllib2
+import glob, md5, os, re, threading, time, urllib, urllib2
 from gettext import gettext as _
 import gobject, gtk
 import xlmisc, library, common
 import xl.path
+from lib import ecs
 
 COVER_WIDTH = 100
 NOCOVER_IMAGE = xl.path.get_data("images", "nocover.png")
 
-__revision__ = ".01"
-
 #LOCALES = ['ca', 'de', 'fr', 'jp', 'uk', 'us']
 
-def get_server(locale):
-    #do 'es' here because webservices.amazon.es doesn't exist
-    if locale in ('en', 'us', 'es'):
-        return "xml.amazon.com"
-    elif locale in ('jp', 'uk'):
-        return "webservices.amazon.co.%s" % locale
-    else:
-        return "webservices.amazon.%s" % locale
-
-def get_encoding(locale):
-    if locale == 'jp':
-        return 'utf-8'
-    else:
-        return 'iso-8859-1'
-
 KEY = "15VDQG80MCS2K1W2VRR2" # Adam Olsen's key (synic)
-QUERY = "/onca/xml3?t=webservices-20&dev-t=%s&mode=music&type=lite&" % (KEY) + \
-    "locale={locale}&page=1&f=xml&KeywordSearch="
-IMAGE_PATTERN = re.compile(
-    r"<ImageUrlLarge>http://(\w+\.images-amazon\.com)"
-    "(/images/.*?\.jpg)</ImageUrlLarge>", re.DOTALL)
+ecs.setLicenseKey(KEY)
 
 """
     Fetches album covers from Amazon.com
@@ -92,7 +72,7 @@ class CoverFetcherThread(threading.Thread):
         self.search_string = search_string
         self.locale = locale
         self.fetch_all = fetch_all
-    
+        ecs.setLocale(locale)
 
     def abort(self):
         """
@@ -107,56 +87,33 @@ class CoverFetcherThread(threading.Thread):
         """
             Actually connects and fetches the covers
         """
-        xlmisc.log("cover thread started")
-        conn = httplib.HTTPConnection(get_server(self.locale))
+        start = time.time()
+        xlmisc.log("Amazon: cover thread started")
 
         if self._done: return
-        try:
-            query = QUERY.replace("{locale}", self.locale)
-            # FIXME: always UTF-8?
-            search_string = self.search_string.decode('utf-8')
-            search_string = search_string.encode(
-                get_encoding(self.locale), 'replace')
-            string = query + urllib.quote(search_string, '')
-        except KeyError:
-            string = ""
-        try:
-            conn.request("GET", string)
-        except urllib2.URLError:
-            pass
-        except:
-            xlmisc.log_exception()
-            pass
+
+        albums = ecs.ItemSearch(Keywords=self.search_string, SearchIndex="Music", 
+            ResponseGroup="ItemAttributes,Images")
+
         if self._done: return
 
-        response = conn.getresponse()
-        if response.status != 200:
-            print dir(response)
-            print response.reason
-            print get_server(self.locale), string
-            xlmisc.log("Invalid response received: %s" % response.status)
-            gobject.idle_add(self._done_func, [])
-            return
-
-        page = response.read()
-
         covers = []
-        for m in IMAGE_PATTERN.finditer(page):
+        for album in albums:
             if self._done: return
 
             cover = Cover()
-
-            conn = httplib.HTTPConnection(m.group(1))
+            # m.group(1) == hostname
+            # m.group(2) == filename
             try:
-                conn.request("GET", m.group(2))
-            except urllib2.URLError:
-                continue
-            response = conn.getresponse()
-            cover['status'] = response.status
-            if response.status == 200:
+                url = album.LargeImage.URL
+            except AttributeError: continue
+            try:
+                response = urllib2.urlopen(url)
+                
+                cover['status'] = 200
+
                 data = response.read()
                 if self._done: return
-                conn.close()
                 cover['data'] = data
                 cover['md5'] = md5.new(data).hexdigest()
 
@@ -164,14 +121,24 @@ class CoverFetcherThread(threading.Thread):
                 if len(data) > 1200:
                     covers.append(cover)
                     if not self.fetch_all: break
-
-        conn.close()
+            except urllib2.HTTPError,e:
+                cover['status'] = e.code
+            except urllib2.URLError:
+                pass
 
         if len(covers) == 0:
-            xlmisc.log("Thread done.... *shrug*, no covers found")
+            xlmisc.log("Amazon: Thread done.... *shrug*, no covers found")
 
         if self._done: return
 
+        end = time.time()
+        if end - start < 1:
+            difference = 1 - (end - start) + .01
+            xlmisc.log("Amazon: sleeping for %s seconds..." % difference)
+        else:
+            difference = .01
+
+        time.sleep(difference)
         # call after the current pending event in the gtk gui
         gobject.idle_add(self._done_func, covers)
 
@@ -330,7 +297,10 @@ class CoverManager(object):
                 if row[1]:
                     if not self.check_image_age(album_id, row[0]): return
 
-                self.exaile.cover.set_image(coverfile)
+                if gtk.gdk.pixbuf_get_file_info(coverfile):
+                    self.exaile.cover.set_image(coverfile)
+                else:
+                    self.exaile.cover.set_image(NOCOVER_IMAGE)
                 return
 
         cover = self.fetch_from_fs(track)
@@ -368,13 +338,13 @@ class CoverManager(object):
         dir = os.path.dirname(track.loc)
 
         names = self.exaile.settings.get_list('art_filenames', 
-            ['cover.jpg', 'folder.jpg', '.folder.jpg', 'album.jpg', 'art.jpg'])
+            [u'cover.jpg', u'folder.jpg', u'.folder.jpg', u'album.jpg', u'art.jpg'])
         if not names: return None
 
         for f in names:
-            f = os.path.join(dir, f.strip())
-            if os.path.isfile(f):
-                return f
+            for f in glob.glob(os.path.join(dir, f.strip())):
+                if os.path.isfile(f):
+                    return f
 
         return None
 
@@ -625,14 +595,18 @@ class CoverFetcher(object):
                 self.needs[artist].append(album)
                 image = NOCOVER_IMAGE
 
-            if self.found.has_key("%s - %s" % (artist.lower(), album.lower())):
-                continue
 
             title = CoverWrapper(artist, album, image)
-            image = gtk.gdk.pixbuf_new_from_file(image)
+            try:
+                image = gtk.gdk.pixbuf_new_from_file(image)
+            except gobject.GError:
+                image = gtk.gdk.pixbuf_new_from_file(NOCOVER_IMAGE)
             image = image.scale_simple(80, 80, 
                 gtk.gdk.INTERP_BILINEAR)
 
+            if self.found.has_key("%s - %s" % (artist.lower(), album.lower())):
+                continue
+
             self.found["%s - %s" % (artist.lower(), album.lower())] = \
                 self.model.append([title, image, title])
             if count >= 200: 
@@ -675,35 +649,136 @@ class CoverFetcher(object):
                 event.button, event.time)
 
 class CoverWindow(object):
-    """
-        Shows the fullsize cover in a window
-    """
+    """Shows the cover in a simple image viewer"""
+
     def __init__(self, parent, cover, title=''):
-        """
-            Initializes and shows the cover
-        """
-        xml = gtk.glade.XML('exaile.glade', 'CoverWindow', 'exaile')
-        window = xml.get_widget('CoverWindow')
-        box = xml.get_widget('cw_box')
-        pixbuf = gtk.gdk.pixbuf_new_from_file(cover)
-        image = gtk.Image()
-        image.set_from_file(cover)
-        image.set_size_request(pixbuf.get_width(), pixbuf.get_height())
-        eventbox = gtk.EventBox()
-        eventbox.add(image)
-        box.pack_start(eventbox)
+        """Initializes and shows the cover"""
+        self.widgets = gtk.glade.XML('exaile.glade', 'CoverWindow', 'exaile')
+        self.widgets.signal_autoconnect(self)
+        self.cover_window = self.widgets.get_widget('CoverWindow')
+        self.layout = self.widgets.get_widget('layout')
+        self.toolbar = self.widgets.get_widget('toolbar')
+        self.zoom_in = self.widgets.get_widget('zoom_in')
+        self.zoom_out = self.widgets.get_widget('zoom_out')
+        self.image = self.widgets.get_widget('image')
+        self.statusbar = self.widgets.get_widget('statusbar')
+        self.scrolledwindow = self.widgets.get_widget('scrolledwindow')
+        self.scrolledwindow.set_hadjustment(self.layout.get_hadjustment())
+        self.scrolledwindow.set_vadjustment(self.layout.get_vadjustment())
+        self.cover_window.set_title(title)
+        self.cover_window.set_transient_for(parent)
+        self.cover_window_width = 500
+        self.cover_window_height = 500 + self.toolbar.size_request()[1] + \
+                                   self.statusbar.size_request()[1]
+        self.scrolledwindow
+        self.cover_window.set_default_size(self.cover_window_width, \
+                                           self.cover_window_height)
+        self.image_original_pixbuf = gtk.gdk.pixbuf_new_from_file(cover)
+        self.image_pixbuf = self.image_original_pixbuf
+        self.min_percent = 1
+        self.max_percent = 500
+        self.ratio = 1.5
+        self.image_interp = gtk.gdk.INTERP_BILINEAR
+        self.image_fitted = True
+        self.set_ratio_to_fit()
+        self.update_widgets()
+        self.cover_window.show_all()
+ 
+    def available_image_width(self):
+        """Returns the available horizontal space for the image"""
+        return self.cover_window.get_size()[0]
 
-        eventbox.connect('button_release_event', self.button_press)
-        window.set_title(title)
-        window.set_transient_for(parent)
-        window.show_all()
-        self.window = window
+    def available_image_height(self):
+        """Returns the available vertical space for the image"""
+        return self.cover_window.get_size()[1] - \
+               self.toolbar.size_request()[1] - \
+               self.statusbar.size_request()[1]
 
-    def button_press(self, widget, event):
-        """
-            Called when someone clicks on the event box
-        """
-        self.window.destroy()
+    def center_image(self):
+        """Centers the image in the layout"""
+        new_x = max(0, int((self.available_image_width() - \
+                            self.image_pixbuf.get_width()) / 2))
+        new_y = max(0, int((self.available_image_height() - \
+                            self.image_pixbuf.get_height()) / 2))
+        self.layout.move(self.image, new_x, new_y)
+
+    def update_widgets(self):
+        """Updates image, layout, scrolled window, tool bar and status bar"""
+        if self.cover_window.window:
+            self.cover_window.window.freeze_updates()
+        self.apply_zoom()
+        self.layout.set_size(self.image_pixbuf.get_width(), \
+                             self.image_pixbuf.get_height())
+        if self.image_fitted or \
+           (self.image_pixbuf.get_width() == self.available_image_width() and \
+           self.image_pixbuf.get_height() == self.available_image_height()):
+            self.scrolledwindow.set_policy(gtk.POLICY_NEVER, gtk.POLICY_NEVER)
+        else:
+            self.scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC,
+                                           gtk.POLICY_AUTOMATIC)
+        percent = int(100 * self.image_ratio)
+        message = str(self.image_original_pixbuf.get_width()) + " x " + \
+                      str(self.image_original_pixbuf.get_height()) + \
+                      " pixels " + str(percent) + '%'
+        self.zoom_in.set_sensitive(percent < self.max_percent)
+        self.zoom_out.set_sensitive(percent > self.min_percent)
+        self.statusbar.pop(self.statusbar.get_context_id(''))
+        self.statusbar.push(self.statusbar.get_context_id(''), message)
+        self.image.set_from_pixbuf(self.image_pixbuf)
+        self.center_image()
+        if self.cover_window.window:
+            self.cover_window.window.thaw_updates()
+
+    def apply_zoom(self):
+        """Scales the image if needed"""
+        new_width = int(self.image_original_pixbuf.get_width() * \
+                        self.image_ratio)
+        new_height = int(self.image_original_pixbuf.get_height() * \
+                         self.image_ratio)
+        if new_width != self.image_pixbuf.get_width() or \
+           new_height != self.image_pixbuf.get_height(): 
+            self.image_pixbuf = self.image_original_pixbuf.scale_simple(new_width, \
+                                  new_height, self.image_interp)
+
+    def set_ratio_to_fit(self):
+        """Calculates and sets the needed ratio to show the full image"""
+        width_ratio = float(self.image_original_pixbuf.get_width()) / \
+                            self.available_image_width()
+        height_ratio = float(self.image_original_pixbuf.get_height()) / \
+                             self.available_image_height()
+        self.image_ratio = 1 / max(1, width_ratio, height_ratio)
+
+    def cover_window_destroy(self, widget):
+        self.cover_window.destroy()
+
+    def zoom_in_clicked(self, widget):
+        self.image_fitted = False
+        self.image_ratio *= self.ratio 
+        self.update_widgets()
+
+    def zoom_out_clicked(self, widget):
+        self.image_fitted = False
+        self.image_ratio *= 1 / self.ratio
+        self.update_widgets()
+
+    def zoom_100_clicked(self, widget):
+        self.image_fitted = False
+        self.image_ratio = 1
+        self.update_widgets()
+
+    def zoom_fit_clicked(self, widget):
+        self.image_fitted = True
+        self.set_ratio_to_fit()
+        self.update_widgets()
+
+    def cover_window_size_allocate(self, widget, allocation):
+        if self.cover_window_width != allocation.width or \
+           self.cover_window_height != allocation.height:
+            if self.image_fitted:
+                self.set_ratio_to_fit()
+            self.update_widgets()
+            self.cover_window_width = allocation.width
+            self.cover_window_height = allocation.height
 
 class CoverFrame(object):
     """
