@@ -20,15 +20,65 @@ pygtk.require('2.0')
 import gtk, gobject, os, signal, sys, thread, pango, gtk.glade
 import xl.dbusinterface, time
 from gettext import gettext as _
-from xl import library, media, audioscrobbler, equalizer, burn, common
+from xl import library, logger, media, audioscrobbler, equalizer, burn, common
 from xl import xlmisc, config, db, covers, player, prefs
 from xl import playlist as playlistmanager
 from xl.plugins import manager as pluginmanager, gui as plugingui
 from xl.gui import playlist as trackslist
 from xl.gui import information
 from xl.panels import collection, radio, playlists, files, device
-import random, gst
-random.seed()
+import random, gst, urllib
+
+def found_updates(exaile, found):
+    message = _("The following plugins have new versions available for install."
+    " You can install them from the plugin manager.\n\n")
+
+    for (name, version) in found:
+        message += "%s\t%s\n" % (name, version)
+
+    common.info(exaile.window, message)
+
+@common.threaded
+def start_updatecheck_thread(playlist_manager):
+    exaile = playlist_manager.exaile
+    # check exaile itself
+    version = map(int, exaile.get_version().replace('devel', 
+        '').replace('b', '').split('.'))
+    check_version = map(int,
+        urllib.urlopen('http://exaile.org/current_version.txt').read().replace('devel', 
+        '').replace('b', '').split('.'))
+
+    if version < check_version:
+        gobject.idle_add(common.info, exaile.window, _("Exaile version %s is "
+            "available.  Grab it from http://www.exaile.org today!") % 
+            '.'.join([str(i) for i in check_version]))
+
+    # check plugins
+    pmanager = exaile.pmanager
+    avail_url = 'http://www.exaile.org/files/plugins/%s/plugin_info.txt' % \
+            exaile.get_plugin_location()
+
+    h = urllib.urlopen(avail_url)
+    lines = h.readlines()
+    h.close()
+
+    found = []
+
+    check = False
+    for line in lines:
+        line = line.strip()
+        (file, name, version, author, description) = line.split('\t')
+        
+        for plugin in pmanager.plugins:
+            if plugin.PLUGIN_NAME == name:
+                installed_ver = map(int, plugin.PLUGIN_VERSION.split('.'))
+                available_ver = map(int, version.split('.'))
+
+                if installed_ver < available_ver:
+                    found.append((name, version))
+
+    if found:
+        gobject.idle_add(found_updates, exaile, found)
 
 class ExaileWindow(gobject.GObject): 
     """
@@ -45,6 +95,7 @@ class ExaileWindow(gobject.GObject):
         'track-information-updated': (gobject.SIGNAL_RUN_LAST, None, ()),
         'quit': (gobject.SIGNAL_RUN_LAST, None, ()),
         'timer_update': (gobject.SIGNAL_RUN_LAST, None, ()),
+        'lastfm_toggle': (gobject.SIGNAL_RUN_LAST, None, (bool,))
     }
     __single = None
 
@@ -92,9 +143,9 @@ class ExaileWindow(gobject.GObject):
         self.dir_queue = []
         self.scan_timer = None
         self.seeking = False
-        self.debug_dialog = xlmisc.DebugDialog(self)
+        self.debug_dialog = logger.init(self, xl.path.get_config('exaile.log'))
         self.col_menus = dict()
-        self.setup_col_menus('track', trackslist.TracksListCtrl.col_map)
+        self.setup_col_menus('track', trackslist.TracksListCtrl.COLUMNS)
         self.plugins_menu = xlmisc.Menu()
         self.rewind_track = 0
         self.player = player.ExailePlayer(self)
@@ -102,6 +153,11 @@ class ExaileWindow(gobject.GObject):
         self.importer = xl.cd_import.CDImporter(self)
         self.audio_disc_page = None
 
+        # check for updates
+        if self.settings.get_boolean('check_for_updates', True):
+            self.playlist_manager.connect('last-playlist-loaded',
+                start_updatecheck_thread)
+
         if self.settings.get_boolean("ui/use_splash", True):
             image = gtk.Image()
             image.set_from_file(xl.path.get_data('images', 'splash.png'))
@@ -144,13 +200,6 @@ class ExaileWindow(gobject.GObject):
         # TRANSLATORS: The title of the main window
         self.window.set_title(_("Exaile Music Player"))
 
-        # log in to audio scrobbler
-        user = self.settings.get_str("lastfm/user", "")
-        password = self.settings.get_crypted("lastfm/pass", "")
-
-        thread.start_new_thread(audioscrobbler.get_scrobbler_session,
-            (self, user, password))
-
         self.playlists_nb = self.xml.get_widget('playlists_nb')
         self.set_tab_placement()
         self.setup_left()
@@ -167,9 +216,23 @@ class ExaileWindow(gobject.GObject):
 
         self.status = xlmisc.StatusBar(self)
 
+        # log in to audio scrobbler
+        user = self.settings.get_str("lastfm/user", "")
+        password = self.settings.get_crypted("lastfm/pass", "")
+
+        thread.start_new_thread(audioscrobbler.get_scrobbler_session,
+            (self, user, password))
+            
+        # Try to load a saved last.fm cache
+        gobject.idle_add(audioscrobbler.load_cache)
+        
         self.playlists_nb.connect('switch-page', self.page_changed)
-        self.playlists_nb.connect('page-added', self.sync_playlists_tabbar)
-        self.playlists_nb.connect('page-removed', self.sync_playlists_tabbar)
+        try:
+            self.playlists_nb.connect('page-added', self.sync_playlists_tabbar)
+            self.playlists_nb.connect('page-removed', self.sync_playlists_tabbar)
+        except TypeError:
+            # Pre-GTK+ 2.10
+            xlmisc.log("Using old GtkNotebook")
         self.playlists_nb.remove_page(0)
 
         self.timer.start()
@@ -180,9 +243,11 @@ class ExaileWindow(gobject.GObject):
             self.stop_button.hide()
         if not self.settings.get_boolean('ui/show_clear_button', True):
             self.clear_button.hide()
+        if not self.settings.get_boolean('ui/show_cover', True):
+            self.xml.get_widget('main_cover_frame').hide()
 
         self.stop_track_button.set_sensitive(False)
-        self.pmanager = pluginmanager.Manager(self) 
+        self.pmanager = pluginmanager.Manager(self, self.update_plugin) 
         enabled_plugins = []
         for k, v in self.settings.get_plugins().iteritems():
             if v:
@@ -201,7 +266,7 @@ class ExaileWindow(gobject.GObject):
                 _("You have not specified any search directories for your "
                 "music library. You may do so now, or choose to do it later.  "
                 "If you want to do it later, you can manage your library "
-                "search directories by going to Tools->Library Manager.  "
+                "search directories by going to Edit->Library Manager.  "
                 "Do you want to choose your library directories now?")) 
             result = dialog.run()
             dialog.destroy()
@@ -219,42 +284,59 @@ class ExaileWindow(gobject.GObject):
         """
             Called when a tag is found in a stream
         """
-        newsong=False
         track = self.player.current
-        if not track or not track.type == 'stream': return True
-        for tag in tags.keys():
-            if tag == 'bitrate': track.bitrate = int(tags[tag])/1000
-            elif tag == 'comment': track.album = tags[tag]
-            elif tag == 'title': 
+        if not (track and track.type == 'stream'): return True
+
+        log = ['Stream tag:']
+        newsong=False
+
+        for key in tags.keys():
+            value = tags[key]
+            try:
+                value = common.to_unicode(value)
+            except UnicodeDecodeError:
+                log.append('  ' + key + " [can't decode]: " + `str(value)`)
+                continue # TODO: What encoding does gst give us?
+
+            log.append('  ' + key + ': ' + value)
+
+            if key == 'bitrate': track.bitrate = int(value) / 1000
+
+            # if there's a comment, but no album, set album to the comment
+            elif key == 'comment' and not track.loc.endswith('.mp3'): track.album = value
+            elif key == 'album': track.album = value
+            elif key == 'artist': track.artist = value
+            elif key == 'duratin': track._len = long(value)
+            elif key == 'track-number': 
                 try:
-                    if track.rawtitle != tags[tag]: 
-                        track.rawtitle=tags[tag]
-                        xlmisc.log("different song")
-                        newsong=True
+                    track.track = int(value)
+                except ValueError: pass
+            elif key == 'genre': track.genre = value
+            elif key == 'title': 
+                try:
+                    if track.rawtitle != value:
+                        track.rawtitle = value
+                        newsong = True
                 except AttributeError:
-                    xlmisc.log("new song")
-                    track.rawtitle=tags[tag]
-                    newsong=True
-                
-                titleArray=(tags[tag]).split(' - ',2)
-                
-                if len(titleArray) > 0:
-                    track.artist = titleArray[0]
+                    track.rawtitle = value
+                    newsong = True
+
+                title_array = value.split(' - ', 1)
+                if len(title_array) == 1 or track.loc.endswith(".mp3"):
+                    track.title = value
                 else:
-                    track.artist = track.rawtitle
-                if len(titleArray) > 1:
-                    track.title = titleArray[1].lstrip()
-                else: 
-                    track.title = track.rawtitle.lstrip()
-            xlmisc.log('%s: %s' % (gst.tag_get_nick(tag), tags[tag]))           
+                    track.artist = title_array[0]
+                    track.title = title_array[1]
+
         self.tracks.refresh_row(track)
         self.update_track_information()
-        xlmisc.log("rawtitle:%s newsong:%s" % (track.title,newsong))
         if newsong:
-            xlmisc.log("asking to fetch cover")
+            log.append('  New song, fetching cover.')
             self.cover_manager.fetch_cover(track)
+
+        xlmisc.log_multi(log)
         return True
-        
+
     def get_version(self):
         """
             Returns the version of Exaile
@@ -265,7 +347,7 @@ class ExaileWindow(gobject.GObject):
         """
             Returns the location of the plugins
         """
-        if sys.modules['__main__'].__version__.find('svn') > -1 \
+        if sys.modules['__main__'].__version__.find('devel') > -1 \
             or sys.modules['__main__'].__version__.find('b') > -1:
             return 'trunk'
         else:
@@ -308,7 +390,7 @@ class ExaileWindow(gobject.GObject):
         self.window.resize(width, height)
         self.window.move(x, y)
 
-    def setup_col_menus(self, pref, map):
+    def setup_col_menus(self, pref, cols):
         """
             Fetches the view column menus from the glade xml definition
         """
@@ -318,30 +400,36 @@ class ExaileWindow(gobject.GObject):
         self.resizable_cols.set_active(self.settings.get_boolean('ui/resizable_cols',
             False))
         self.resizable_cols.connect('activate', self.activate_cols_resizable)
+        self.not_resizable_cols.set_active(not self.settings.get_boolean('ui/resizable_cols',
+                    False))
         self.not_resizable_cols.connect('activate',
             self.activate_cols_resizable)
 
-        # setup up default shown columns
-        if not self.settings.get_boolean('ui/trackslist_defaults_set', False):
-            self.settings.set_boolean('ui/trackslist_defaults_set', True)
-            columns = []
-            for col in trackslist.TracksListCtrl.default_columns:
-                columns.append(col)
-            self.settings.set_list('ui/%s_columns' % (pref,), columns)
+        column_ids = None
+        if self.settings.get_boolean('ui/trackslist_defaults_set', False):
+            column_ids = set()
+            ids = self.settings.get_list("ui/%s_columns" % pref)
+            # Don't add invalid columns.
+            all_ids = frozenset(trackslist.TracksListCtrl.COLUMN_IDS)
+            for id in ids:
+                if id in all_ids:
+                    column_ids.add(id)
+        if not column_ids:
+            # Use default.
+            ids = trackslist.TracksListCtrl.default_column_ids
+            self.settings.set_boolean('ui/new_trackslist_defaults_set', True)
+            self.settings.set_list('ui/%s_columns' % pref, ids)
+            column_ids = frozenset(ids)
 
-        self.col_menus[pref] = dict()
-        column_settings = self.settings.get_list('ui/%s_columns' % (pref,))
+        self.col_menus[pref] = {}
 
-        for k, v in map.iteritems():
-            self.col_menus[v] = self.xml.get_widget('%s_%s_col' % (pref,
-                v))
-            show = False
-            if k in column_settings:
-                show = True
+        for col_struct in cols:
+            self.col_menus[col_struct.id] = menu = self.xml.get_widget(
+                '%s_%s_col' % (pref, col_struct.id))
 
-            self.col_menus[v].set_active(show)
-            self.col_menus[v].connect('activate', 
-                self.change_column_settings, {'key': 'ui/%s_columns' % (pref,), 'value': k})
+            menu.set_active(col_struct.id in column_ids)
+            menu.connect('activate', self.change_column_settings,
+                ('ui/%s_columns' % pref, col_struct))
 
     def activate_cols_resizable(self, widget, event=None):
         """
@@ -359,17 +447,19 @@ class ExaileWindow(gobject.GObject):
         """
             Changes column view settings
         """
-        columns = self.settings.get_list(data['key'])
-        columns = list(columns)
+        pref, col_struct = data
+        id = col_struct.id
+
+        column_ids = list(self.settings.get_list(pref))
         if item.get_active():
-            if data['value'] not in columns:
-                xlmisc.log("adding %s column to %s" % (data['value'], data['key']))
-                columns.append(data['value'])
+            if id not in column_ids:
+                xlmisc.log("adding %s column to %s" % (id, pref))
+                column_ids.append(id)
         else:
-            if data['value'] in columns:
-                xlmisc.log("removing %s column from %s" % (data['value'], data['key']))
-                columns.remove(data['value'])
-        self.settings.set_list(data['key'], columns)
+            if col_struct.id in column_ids:
+                xlmisc.log("removing %s column from %s" % (id, pref))
+                column_ids.remove(id)
+        self.settings.set_list(pref, column_ids)
 
         for i in range(0, self.playlists_nb.get_n_pages()):
             page = self.playlists_nb.get_nth_page(i)
@@ -487,6 +577,11 @@ class ExaileWindow(gobject.GObject):
         key, mod = gtk.accelerator_parse('<Control>L')
         self.accel_group.connect_group(key, mod, gtk.ACCEL_VISIBLE,
             lambda *e: self.jump_to(3))
+
+        # toggle last.fm submissions
+        key, mod = gtk.accelerator_parse('<Control><Shift>L')
+        self.accel_group.connect_group(key, mod, gtk.ACCEL_VISIBLE,
+            lambda *e: self.toggle_lastfm())
         
         # add the accel group to the window
         self.window.add_accel_group(self.accel_group)
@@ -500,8 +595,8 @@ class ExaileWindow(gobject.GObject):
         self.goto_current_item.connect('activate', self.goto_current)
 
         self.xml.get_widget('about_item').connect('activate',
-            lambda *e: xlmisc.AboutDialog(self.window,
-            sys.modules['__main__'].__version__))
+            lambda *e: xlmisc.show_about_dialog(self.window,
+            self.get_version_info()))
         
         self.xml.get_widget('new_item').connect('activate',
             lambda *e: self.new_page())
@@ -544,6 +639,35 @@ class ExaileWindow(gobject.GObject):
         self.stop_track_button.connect('clicked',
             self.stop_track_toggle)
 
+    def toggle_lastfm(self):
+        """
+            Toggles last.fm submissions
+        """
+        new_setting = not self.settings.get_boolean('lastfm/submit', True)
+        self.settings.set_boolean('lastfm/submit', new_setting)
+        xlmisc.log("Toggling last.fm submissions to: %s" % new_setting)
+        self.emit('lastfm_toggle', new_setting)
+
+        type = _('Enabling')
+        if not new_setting: type = _('Disabling')
+        message = type + ' ' + _('Last.FM Submissions...')
+        self.status.set_first(message, 2000)
+
+    def get_version_info(self):
+        """
+            Gets the current version information
+            If this is a development version, add on the revision number.
+            This number is in xl/version.py, which is generated when the
+            "make" command is issued (via bzr version-info --format=python >
+            xl/version.py
+        """
+        version = sys.modules['__main__'].__version__
+        if 'devel' in version:
+            import xl.version
+            version = "%s [r%s]" % (version, xl.version.version_info['revno'])
+
+        return version
+
     def stop_track_toggle(self, *e):
         """
             Stop track toggle
@@ -624,8 +748,8 @@ class ExaileWindow(gobject.GObject):
         """
             Shows the debug dialog if it has been initialized
         """
-        if xlmisc.DebugDialog.debug:
-            xlmisc.DebugDialog.debug.dialog.show()
+        if logger.gui:
+            logger.gui.dialog.show()
 
     def live_search(self, *e):
         """
@@ -946,9 +1070,9 @@ class ExaileWindow(gobject.GObject):
                 self.db.add_function_create(('THE_CUTTER', 1, 
                     library.the_cutter))
             except:
-                pass # db is ok, continue!                
+                pass # db is ok, continue!
 
-        self.db.check_version("sql")
+        self.db.check_version(xl.path.get_data('sql'))
 
 
     def initialize(self):
@@ -1012,17 +1136,28 @@ class ExaileWindow(gobject.GObject):
             Updates the seeker position, the "now playing" title, and
             submits the track to last.fm when appropriate
         """
-        self.status.set_track_count(_("%(track_count)d showing, "
-            "%(track_total)d in collection") %
-            {
-                'track_count': len(self.songs),
-                'track_total': len(self.all_songs)
-            })
+        status_text = ""
+        track_count = len(self.songs)
+
+        if track_count:
+            #TRANSLATORS: Number of tracks in the playlist
+            status_text += _("%d showing") % track_count
+
+            total_time = self.songs.get_total_length()
+            if total_time:
+                status_text += " (" + total_time + ")"
+
+            status_text += ", "
+
+        #TRANSLATORS: Number of tracks in the collection
+        status_text += _("%d in collection") % len(self.all_songs)
+        self.status.set_track_count(status_text)
+
         track = self.player.current
 
         self.rewind_track += 1
 
-        if track == None: 
+        if track is None: 
             return True
         duration = track.duration
 
@@ -1054,12 +1189,15 @@ class ExaileWindow(gobject.GObject):
 
 
         if (seconds > 240 or value > 50) and track.type != 'stream' and \
-            self.player.is_playing() and not track.submitted: 
+            track.type != 'podcast' and self.player.is_playing() \
+            and not track.submitted: 
             track.submitted = True
             self.update_rating(track, plays=1,
                 rating=1)
             self.status.set_first(_("Submitting to Last.fm..."), 2000)
-            audioscrobbler.submit_to_scrobbler(self, track)
+
+            if self.settings.get_boolean('lastfm/submit', True):
+                audioscrobbler.submit_to_scrobbler(self, track)
 
         self.emit('timer_update')
 
@@ -1125,6 +1263,7 @@ class ExaileWindow(gobject.GObject):
             rating = row[1]
             if rating <= 0 or rating == '' or rating is None: 
                 rating = 0
+
             self.rating_combo.set_active(rating - 1)
             track.user_rating = rating
             self.rating_combo.set_sensitive(True)
@@ -1152,6 +1291,10 @@ class ExaileWindow(gobject.GObject):
         self.db.execute("UPDATE tracks SET %s WHERE path=?" % update_string, 
             (path_id,))
 
+        track.playcount += plays
+
+        self.tracks.refresh_row(track)
+
     
     def setup_right(self): 
         """
@@ -1262,7 +1405,7 @@ class ExaileWindow(gobject.GObject):
             Called when something is typed into the filter box
         """
 
-        keyword = self.tracks_filter.get_text()
+        keyword = unicode(self.tracks_filter.get_text(), 'utf-8')
         if keyword.startswith("where ") and not widget: return
         self.songs = library.search(self, self.tracks.playlist_songs, None,
             custom=custom)
@@ -1451,6 +1594,7 @@ class ExaileWindow(gobject.GObject):
             xl.prefs.TEXT_VIEW_DEFAULT)
         pop.show_track_osd(track, text_display,
             cover)
+        self.timer_update()
 
     def setup_menus(self):
         """
@@ -1504,23 +1648,27 @@ class ExaileWindow(gobject.GObject):
         dialog.set_current_folder(self.get_last_dir())
         dialog.set_select_multiple(True)
 
-        # Media Files
-        filter = gtk.FileFilter()
-        filter.set_name(_("Media Files"))
+        supported = gtk.FileFilter()
+        supported.set_name(_("Supported Files"))
+        music = gtk.FileFilter()
+        music.set_name(_("Music Files"))
+        playlist = gtk.FileFilter()
+        playlist.set_name(_("Playlist Files"))
+        all = gtk.FileFilter()
+        all.set_name(_("All Files"))
+
         for ext in media.SUPPORTED_MEDIA:
-            filter.add_pattern('*' + ext)
-        dialog.add_filter(filter)
-        filter = gtk.FileFilter()
-        # Playlist Files
-        filter.set_name(_("Playlist Files"))
+            supported.add_pattern('*' + ext)
+            music.add_pattern('*' + ext)
         for ext in xlmisc.PLAYLIST_EXTS:
-            filter.add_pattern('*' + ext)
-        dialog.add_filter(filter)
-        # All Files
-        filter = gtk.FileFilter()
-        filter.set_name(_("All Files"))
-        filter.add_pattern('*')
-        dialog.add_filter(filter)
+            supported.add_pattern('*' + ext)
+            playlist.add_pattern('*' + ext)
+        all.add_pattern('*')
+
+        dialog.add_filter(supported)
+        dialog.add_filter(music)
+        dialog.add_filter(playlist)
+        dialog.add_filter(all)
 
         result = dialog.run()
         dialog.hide()
@@ -1642,6 +1790,10 @@ class ExaileWindow(gobject.GObject):
         # PLUGIN: send plugins event before quitting
         self.emit('quit')
 
+        # Write any tracks remaining in the last.fm cache to disk
+        # for submission later.
+        audioscrobbler.write_cache()
+
         self.player.stop()
         self.cover_manager.stop_cover_thread()
         for thread in self.thread_pool:
@@ -1659,8 +1811,16 @@ class ExaileWindow(gobject.GObject):
         queuefile = xl.path.get_config('queued.save')
         if os.path.isfile(queuefile):
             os.unlink(queuefile)
+            
 
         if self.player.current: self.player.current.stop()
+        # Clear search filter if needed so that the entire playlist is saved
+        if self.tracks_filter.get_text() != '':
+            self.tracks_filter.set_text('')
+            try:
+                self.on_search()
+            except:  # In case we're quitting before the playlist loaded
+                pass
 
         for i in range(self.playlists_nb.get_n_pages()):
             page = self.playlists_nb.get_nth_page(i)
