@@ -14,7 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-import thread, os, shlex, string
+import thread, os, shlex, string, urllib2
 from gettext import gettext as _
 import pygtk, common
 pygtk.require('2.0')
@@ -63,15 +63,18 @@ class PrefsItem(object):
             Sets up the function to be called when this preference is changed
         """
         self.widget.connect('focus-out-event',
-            self.change, self.name, self.widget.get_text())
+            self.change, self.name, unicode(self.widget.get_text(), 'utf-8'))
         self.widget.connect('activate',
             lambda *e: self.change(self.widget, None, self.name,
-                self.widget.get_text()))
+                unicode(self.widget.get_text(), 'utf-8')))
 
     def set_pref(self):
         """ 
             Sets the GUI widget up for this preference
         """
+        if not self.widget:
+            xlmisc.log("Widget not found: %s" % (self.name))
+            return
         self.widget.set_text(str(settings.get_str(self.name, default=self.default)))
 
     def do_done(self):
@@ -85,7 +88,7 @@ class PrefsItem(object):
             applies this setting
         """
         if self.done and not self.do_done(): return False
-        settings.set_str(self.name, self.widget.get_text())
+        settings.set_str(self.name, unicode(self.widget.get_text(), 'utf-8'))
         return True
 
 class CryptedPrefsItem(PrefsItem):
@@ -101,7 +104,7 @@ class CryptedPrefsItem(PrefsItem):
 
     def apply(self):
         if self.done and not self.do_done(): return False
-        settings.set_crypted(self.name, self.widget.get_text())
+        settings.set_crypted(self.name, unicode(self.widget.get_text(), 'utf-8'))
         return True
 
 class PrefsTextViewItem(PrefsItem):
@@ -128,7 +131,7 @@ class PrefsTextViewItem(PrefsItem):
         buf = self.widget.get_buffer()
         start = buf.get_start_iter()
         end = buf.get_end_iter()
-        return buf.get_text(start, end)
+        return unicode(buf.get_text(start, end), 'utf-8')
 
     def set_pref(self):
         """
@@ -180,14 +183,26 @@ class ListPrefsItem(PrefsItem):
 
     def set_pref(self):
         items = settings.get_list(self.name, default=self.default)
-        self.widget.set_text(" ".join(items))
+        try:
+            items = " ".join(items)
+        except:
+            items = ""
+        self.widget.set_text(items)
 
     def apply(self):
         if self.done and not self.do_done(): return False
-        text = self.widget.get_text()
-        settings.set_list(self.name, shlex.split(text))
+        # shlex is broken with unicode, so we feed it UTF-8 and encode
+        # afterwards.
+        values = shlex.split(self.widget.get_text())
+        values = [unicode(value, 'utf-8') for value in values]
+        settings.set_list(self.name, values)
         return True
 
+class SpinPrefsItem(PrefsItem):
+    def set_pref(self):
+        value = settings.get_float(self.name, default=self.default)
+        self.widget.set_value(value)
+
 class FloatPrefsItem(PrefsItem):
     """
         A class to represent a floating point number in the preferences window
@@ -260,7 +275,7 @@ class DirPrefsItem(PrefsItem):
         """
             Sets the current directory
         """
-        directory = settings.get_str(self.name, self.default)
+        directory = os.path.expanduser(settings.get_str(self.name, self.default))
         if not os.path.exists(directory):
             os.makedirs(directory)
         self.widget.set_filename(directory)
@@ -335,6 +350,8 @@ class Preferences(object):
         # TRANSLATORS: Category of the preferences dialog
         (_("Advanced"), [
             _("Replay Gain"),
+            # TRANSLATORS: Category of the preferences dialog
+            _("Proxy Settings")
         ]),
     ]
 
@@ -392,7 +409,6 @@ class Preferences(object):
         xml.get_widget('prefs_lastfm_pass').set_invisible_char('*')
         xml.get_widget('prefs_audio_sink').set_active(0)
 
-
         self.setup_settings()
 
     def setup_settings(self):
@@ -463,7 +479,11 @@ class Preferences(object):
         # replaygain toggle handler
         replaygain = xml.get_widget('prefs_replaygain_enabled')
         replaygain.connect('toggled', self.toggle_replaygain)
-        
+
+        # proxy server handler
+        proxy = xml.get_widget('prefs_proxy_enabled')
+        proxy.connect('toggled', self.toggle_proxy)
+
         simple_settings = ({
             'ui/use_splash': (CheckPrefsItem, True),
 #            'watch_directories': (CheckPrefsItem, False, self.check_gamin,
@@ -481,6 +501,7 @@ class Preferences(object):
             'osd/h': (PrefsItem, '95', self.osd_adjust_size),
             'lastfm/user': (PrefsItem, ''),
             'lastfm/pass': (CryptedPrefsItem, '', None, self.setup_lastfm),
+            'lastfm/submit': (CheckPrefsItem, True),
             'cd_device': (PrefsItem, '/dev/cdrom'),
             'audio_sink': (ComboPrefsItem, 'Use GConf Settings'), # FIXME: i18n?
             'burn_prog': (ComboPrefsItem, (burn_progs and burn_progs[0]) or ''),
@@ -491,12 +512,13 @@ class Preferences(object):
                 self.osd_colorpicker),
             'osd/text_font': (FontButtonPrefsItem, 'Sans 10',
                 self.osd_fontpicker),
+            'osd/opacity': (SpinPrefsItem, 80, self.osd_adjust_opacity),
             'ui/use_tray': (CheckPrefsItem, False, None, self.setup_tray),
             'amazon_locale': (ComboPrefsItem, 'us'),
             'wikipedia_locale': (PrefsItem, 'en'),
             'scan_interval': (FloatPrefsItem, 25, None,
                 self.setup_scan_interval),
-            'download_feeds': (CheckPrefsItem, True),
+            'feed_download_location': (DirPrefsItem, '~/.exaile/podcasts'),
             'import/format': (ComboPrefsItem, 
                 (import_formats and import_formats[0]) or 'MP3'),
             'import/quality': (ComboPrefsItem, "High"), # FIXME: i18n?
@@ -508,6 +530,12 @@ class Preferences(object):
             'replaygain/album_mode': (CheckPrefsItem, True),
             'replaygain/preamp': (FloatPrefsItem, 0.0),
             'replaygain/fallback': (FloatPrefsItem, 0.0),
+            'check_for_updates': (CheckPrefsItem, True),
+            'proxy/enabled': (CheckPrefsItem, False, None, self.proxy_setting_changed),
+            'proxy/server': (PrefsItem, '', None, self.proxy_setting_changed),
+            'proxy/port': (PrefsItem, '', None, self.proxy_setting_changed),
+            'proxy/username': (PrefsItem, '', None, self.proxy_setting_changed),
+            'proxy/password': (CryptedPrefsItem, '', None, self.proxy_setting_changed),
         })
 
         for setting, value in simple_settings.iteritems():
@@ -523,6 +551,12 @@ class Preferences(object):
 
     setting_changed = setup_settings
 
+    def advanced_toggle_cover(self, item, value):
+        if value:
+            self.exaile.xml.get_widget('main_cover_frame').show_all()
+        else:
+            self.exaile.xml.get_widget('main_cover_frame').hide()
+
     def advanced_toggle_tabbar(self, item, value):
         # value == always show
         self.exaile.playlists_nb.set_show_tabs(
@@ -602,12 +636,41 @@ class Preferences(object):
             bitrate_label.show()
 
 
+    def proxy_setting_changed(self, widget):
+        """
+            Apply proxy settings
+        """
+        if self.exaile.settings.get_boolean('proxy/enabled', False):
+            server = self.exaile.settings.get_str('proxy/server', "")
+            port = self.exaile.settings.get_str('proxy/port', "")
+            username = self.exaile.settings.get_str('proxy/username', "")
+            password = self.exaile.settings.get_str('proxy/password', "")
+
+            proxy_url = "http://"
+
+            if username != "" and password != "":
+                proxy_url += "%s:%s@" % (username, password)
+
+            proxy_url += server
+
+            if port != "":
+                proxy_url += ":%s" % port
+
+            proxy_support = urllib2.ProxyHandler({'http':proxy_url})
+            opener = urllib2.build_opener(proxy_support, urllib2.HTTPHandler)
+            urllib2.install_opener(opener)
+        else:
+            opener = urllib2.build_opener(urllib2.HTTPHandler)
+            urllib2.install_opener(opener)
+
+        return True
+
     def import_location_changed(self, widget):
         """
             Scan the new import location for new songs
         """
         items = self.exaile.settings.get_list('search_paths', '')
-        path = widget.get_text()
+        path = unicode(widget.get_text(), 'utf-8')
         if not path: return True
 
         new_items = []
@@ -668,6 +731,14 @@ class Preferences(object):
         for item in items:
             item.set_sensitive(to_state)
 
+    def toggle_proxy(self, widget, event=None):
+        """
+            Enables/disables proxy server options.
+        """
+        item = xml.get_widget('prefs_proxy_table')
+        to_state = widget.get_active()
+        item.set_sensitive(to_state)
+
     def use_custom_toggled(self, widget, event=None):
         """
             Set sensitivity of widgets when checkbox is toggled
@@ -689,7 +760,7 @@ class Preferences(object):
         #naming = xml.get_widget('prefs_import_naming')
 
         #dir = location.get_filename()
-        path = widget.get_text()
+        path = unicode(widget.get_text(), 'utf-8')
 
         # TODO: get proper mapping
         mapping = {'artist':'Artist', 'album':'Album', 'num':1, 
@@ -709,9 +780,11 @@ class Preferences(object):
         """
             Connects to last.fm if the password field isn't empty
         """
-        if not widget.get_text(): return True
-        user = xml.get_widget('prefs_lastfm_user').get_text()
+        # The audioscrobbler module can handle unicode or UTF-8 str;
+        # no need to decode.
         password = widget.get_text()
+        if not password: return True
+        user = xml.get_widget('prefs_lastfm_user').get_text()
 
         thread.start_new_thread(audioscrobbler.get_scrobbler_session,
             (self.exaile, user, password, True))
@@ -784,6 +857,19 @@ class Preferences(object):
         self.osd_settings[name] = val
         self.display_popup()
 
+    def osd_adjust_opacity(self, widget, event, name, previous):
+        """
+            Called when the user requests to adjust the opacity (alpha value) of the osd
+        """
+        try:
+            val = int(widget.get_text())
+        except ValueError:
+            widget.set_text(previous)
+            return
+
+        self.osd_settings[name] = val
+        self.display_popup()
+
     def osd_fontpicker(self, widget, name):
         """
             Gets the font from the font picker, and re-sets up the OSD window
